@@ -60,12 +60,19 @@ import Control.Applicative
 import Data.Typeable (Typeable)
 import Data.Int (Int8, Int16, Int32, Int64)
 import Data.Word (Word8, Word16, Word32, Word64)
+
+#if MIN_VERSION_blaze_html(0,5,0)
+import Text.Blaze.Html
+import Text.Blaze.Internal (preEscapedText)
+import Text.Blaze.Html.Renderer.Text (renderHtml)
+#else
 import Text.Blaze (Html, preEscapedText)
 import Text.Blaze.Renderer.Text (renderHtml)
+#endif
+
 import qualified Data.Text as T
 import qualified Data.Text.Lazy as TL
 import qualified Data.ByteString.Lazy as L
-import qualified Control.Monad.IO.Class as Trans
 
 import qualified Control.Exception as E
 import Control.Monad.Trans.Error (Error (..))
@@ -79,7 +86,6 @@ import qualified Data.Text.Encoding as T
 import qualified Data.Text.Encoding.Error as T
 import Web.PathPieces (PathPiece (..))
 import qualified Data.Text.Read
-import qualified Data.Conduit as C
 
 import Data.Aeson (Value)
 import Data.Aeson.Types (Parser)
@@ -99,6 +105,10 @@ import Data.Aeson.Encode (fromValue)
 import Data.Text.Lazy (toStrict)
 import Data.Text.Lazy.Builder (toLazyText)
 
+import Control.Monad.Trans.Control (MonadBaseControl)
+import Control.Monad.Base (liftBase)
+import Control.Monad.IO.Class (MonadIO)
+
 data PersistException
   = PersistError T.Text -- ^ Generic Exception
   | PersistMarshalError T.Text
@@ -396,8 +406,9 @@ class PersistEntity val where
 instance PersistField a => PersistField [a] where
     toPersistValue = PersistList . map toPersistValue
     fromPersistValue (PersistList l) = fromPersistList l
-    fromPersistValue (PersistText t)
-        | Just values <- A.decode' (L.fromChunks [TE.encodeUtf8 t]) = fromPersistList values
+    fromPersistValue (PersistText t) = fromPersistValue (PersistByteString $ TE.encodeUtf8 t)
+    fromPersistValue (PersistByteString bs)
+        | Just values <- A.decode' (L.fromChunks [bs]) = fromPersistList values
     fromPersistValue x = Left $ "Expected PersistList, received: " ++ show x
     sqlType _ = SqlString
 
@@ -405,10 +416,11 @@ instance (Ord a, PersistField a) => PersistField (S.Set a) where
     toPersistValue = PersistList . map toPersistValue . S.toList
     fromPersistValue (PersistList list) =
       either Left (Right . S.fromList) $ fromPersistList list
-    fromPersistValue (PersistText t)
-        | Just values <- A.decode' (L.fromChunks [TE.encodeUtf8 t]) =
+    fromPersistValue (PersistText t) = fromPersistValue (PersistByteString $ TE.encodeUtf8 t)
+    fromPersistValue (PersistByteString bs)
+        | Just values <- A.decode' (L.fromChunks [bs]) =
             either Left (Right . S.fromList) $ fromPersistList values
-    fromPersistValue x = Left $ "Expected PersistList, received: " ++ show x
+    fromPersistValue x = Left $ "Expected PersistSet, received: " ++ show x
     sqlType _ = SqlString
 
 fromPersistList :: PersistField a => [PersistValue] -> Either T.Text [a]
@@ -431,8 +443,9 @@ instance PersistField v => PersistField (M.Map T.Text v) where
 
 getPersistMap :: PersistValue -> Either T.Text [(T.Text, PersistValue)]
 getPersistMap (PersistMap kvs) = Right kvs
-getPersistMap (PersistText t)
-    | Just pairs <- A.decode' (L.fromChunks [TE.encodeUtf8 t]) = Right pairs
+getPersistMap (PersistText t)  = getPersistMap (PersistByteString $ TE.encodeUtf8 t)
+getPersistMap (PersistByteString bs)
+    | Just pairs <- A.decode' (L.fromChunks [bs]) = Right pairs
 getPersistMap x = Left $ "Expected PersistMap, received: " ++ show x
 
 fromPersistMap :: PersistField v
@@ -500,7 +513,7 @@ data Entity entity =
            , entityVal :: entity }
     deriving (Eq, Ord, Show, Read)
 
-class (C.ResourceIO m, C.ResourceIO (b m)) => PersistStore b m where
+class (MonadBaseControl IO m, MonadBaseControl IO (b m)) => PersistStore b m where
 
     -- | Create a new record in the database, returning an automatically created
     -- key (in SQL an auto-increment id).
@@ -540,7 +553,7 @@ class PersistStore b m => PersistUnique b m where
     insertUnique :: (b ~ PersistEntityBackend val, PersistEntity val) => val -> b m (Maybe (Key b val))
     insertUnique datum = do
         isUnique <- checkUnique datum
-        if isUnique then Just <$> insert datum else return Nothing
+        if isUnique then Just `liftM` insert datum else return Nothing
 
 
 
@@ -596,7 +609,7 @@ belongsToJust getForeignKey model = getJust $ getForeignKey model
 --   Unsafe unless your database is enforcing that the foreign key is valid
 getJust :: (PersistStore b m, PersistEntity val, Show (Key b val)) => Key b val -> b m val
 getJust key = get key >>= maybe
-  (Trans.liftIO $ E.throwIO $ PersistForeignConstraintUnmet $ show key)
+  (liftBase $ E.throwIO $ PersistForeignConstraintUnmet $ show key)
   return
 
 
@@ -647,7 +660,9 @@ class PersistConfig c where
     createPoolConfig :: c -> IO (PersistConfigPool c)
 
     -- | Run a database action by taking a connection from the pool.
-    runPool :: C.ResourceIO m => c -> PersistConfigBackend c m a
+    runPool :: (MonadBaseControl IO m, MonadIO m)
+            => c
+            -> PersistConfigBackend c m a
             -> PersistConfigPool c
             -> m a
 
